import { BrowserModule } from '@angular/platform-browser';
import { NgModule, APP_INITIALIZER } from '@angular/core';

import { AppComponent } from './app.component';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { MatMenuModule } from '@angular/material/menu';
import { MatDividerModule } from '@angular/material/divider';
import { MatCardModule } from '@angular/material/card';
import { MatInputModule } from '@angular/material/input';
import { MatDialogModule } from '@angular/material/dialog';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatSelectModule } from '@angular/material/select';
import { MatSidenavModule } from '@angular/material/sidenav';

import { RepositoryComponent } from './repository/repository.component';
import { AccessapiComponent } from './accessapi/accessapi.component';
import { DatasetsComponent } from './datasets/datasets.component';
import { SettingfdpComponent } from './settingfdp/settingfdp.component';
import { SettingsmartapiComponent } from './settingsmartapi/settingsmartapi.component';
import { UserComponent } from './user/user.component';

import { ReactiveFormsModule } from '@angular/forms';
import { FormsModule } from '@angular/forms';

import { RepositoryinfoComponent } from './repositoryinfo/repositoryinfo.component';

import { HttpClientModule, HttpClient } from '@angular/common/http';
import { FileSaverModule } from 'ngx-filesaver';
import { AppConfiguration } from './AppConfiguration';
import { PublishApiComponent } from './publishapi/publishapi.component';
import { SearchComponent } from './search/search.component'
import { ParseXmlService } from './services/parse-xml.service';
import { AuthenticationModule } from './authentication/authentication.module';
import { SearchModule } from './search/search.module';
import { StatsComponent } from './stats/stats.component';
import { MappingComponent } from './mapping/mapping.component';
import { NebularModule } from './nebular.module';

import { AppRoutingModule } from './app-routing.module';
import { NbLayoutModule, NbThemeModule, NbTooltipModule, NbSpinnerModule, NbSelectModule, NbTabsetModule, NbAutocompleteModule, NbListModule, NbAccordionModule, NbDialogModule } from '@nebular/theme';

import { DashboardComponent } from './dashboard/dashboard.component';
import { FeedbackDialogComponent } from './mapping/dialog/feedback-dialog/feedback-dialog.component';
import { NbToastrModule } from '@nebular/theme';
import { DatasetsDialogComponent } from './datasets/datasets-dialog/datasets-dialog.component';





@NgModule({
  declarations: [
    AppComponent,
    RepositoryComponent,
    AccessapiComponent,
    DatasetsComponent,
    SettingfdpComponent,
    SettingsmartapiComponent,
    UserComponent,
    RepositoryinfoComponent,
    PublishApiComponent,
    SearchComponent,
    StatsComponent,
    DashboardComponent,
    MappingComponent,
    FeedbackDialogComponent,
    DatasetsDialogComponent,
  ],
  imports: [
    BrowserModule,
    BrowserAnimationsModule,
    MatMenuModule,
    MatDividerModule,
    MatCardModule,
    MatInputModule,
    MatDialogModule,
    MatProgressSpinnerModule,
    MatSelectModule,
    MatSidenavModule,
    FormsModule,
    ReactiveFormsModule,
    HttpClientModule,
    FileSaverModule,
    SearchModule,
    AppRoutingModule,
    AuthenticationModule,
    NebularModule,
    BrowserAnimationsModule,
    NbThemeModule.forRoot({ name: 'default' }),
    NbLayoutModule,
    NbSpinnerModule,
    NbSelectModule,
    NbTabsetModule,
    NbTooltipModule,
    NbAutocompleteModule,
    NbListModule,
    NbAccordionModule,
    NbToastrModule.forRoot(),
    NbDialogModule.forRoot()
  ],
  entryComponents: [DatasetsDialogComponent, FeedbackDialogComponent],
  providers: [
    AppConfiguration,
    ParseXmlService,
    {
      provide: APP_INITIALIZER,
      useFactory: AppConfigurationFactory,
      deps: [AppConfiguration, HttpClient], multi: true
    },
  ],
  bootstrap: [AppComponent]
})
export class AppModule { }

export function AppConfigurationFactory(appConfig: AppConfiguration) {
  return () => appConfig.ensureInit();
}
