

import { HttpResponse } from '@angular/common/http';
import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Router } from '@angular/router';
import { FileSaverService } from 'ngx-filesaver';

import { Observable, of, Subscription } from 'rxjs';
import { map } from 'rxjs/operators';
import { environment } from 'src/environments/environment.prod';
import { DatasetCrudService } from '../datasets/services/dataset-crud.service';

import { Dataset, DatasetPath } from './class/dataset';
import { FeedbackDialogComponent } from './dialog/feedback-dialog/feedback-dialog.component';


@Component({
  selector: 'app-mapping',
  templateUrl: './mapping.component.html',
  styleUrls: ['./mapping.component.scss']
})
export class MappingComponent implements OnInit {

  ontology: string;
  itemsdataset: Object[] = []
  dcatVocabulary: Dataset[];
  vocabularies: Dataset[];
  filteredOptions: Observable<string[]>;
  keys: string[] = [];
  selectedPaths: string[];
  mappedMetadatas: Map<number, string>[] = [];
  DatasetToPublish: Map<string, string>[];
  index: number = 0
  first: boolean = true;
  loading: boolean = false;
  loadingCr = false;
  FDP_URL = environment.fdpUrl;
  Object: Object;
  ids: number[];
  @ViewChild('autoInput') input;
  @Input() type: string;
  @Input() catalogId: any;

  constructor(private dataSetService: DatasetCrudService, private dialog: MatDialog, private route: Router, private fileSaverService: FileSaverService) { }

  ngOnInit() {
    this.itemsdataset = this.dataSetService.itemsDataset;
    this.ids = this.dataSetService.ids;
    this.ids
    this.keys = [];
    this.type == "Dataverse" ? this.getKeysFromMetadataDataverse(this.itemsdataset[0], '') :  this.getKeysFromMetadataCustom(this.itemsdataset[0], '');
    this.keys = Array.from(new Set(this.keys));
    console.table(this.keys)
    this.filteredOptions = of(this.keys);
  }

  selectOntology(): void {
    
      this.dataSetService.getLocally('./assets/'+ this.ontology + '.json').subscribe(
        dataset => {
          this.dcatVocabulary = dataset;
          this.vocabularies = dataset;
          this.selectedPaths = [];
          
        },
        error => {
          console.error(error);
        },
      );
  }


  createDataset(item: Object): Map<number, string> {
    console.log("rerereer" + this.selectedPaths)
    let mappedMetadata: Map<number, string> = new Map()
    for (let i = 0; i < this.selectedPaths.length; i++) {
      if (this.selectedPaths[i]) {
        
        let tab = this.selectedPaths[i].split(' : ');
        this.type == "Dataverse" ? mappedMetadata.set(i, this.getValueDataverse(tab, item)) : mappedMetadata.set(i, this.getValueCustom(tab, item));
      }
    }
    this.loadingCr = false;
    return mappedMetadata;
  }

  publishDataset() {
    let version = false;
    let data: string = '';
    let properties: string = '';
    let postedDatastes = [];
    let notPostedDatasets = [];
    this.loading = true;
    const requestPromises: Promise<any>[] = [];
    for (let i = 0; i < this.mappedMetadatas.length; i++) {
      properties = "";
      this.mappedMetadatas[i].forEach((value: string, index: number) => {
        let key = this.dcatVocabulary[index].identifier;
        if (key === 'dct:hasVersion') {
          version = true;
        }
        if (Array.isArray(value)) {
          if (this.isMultiple(key)) {
            value.forEach(v => {
              properties += this.writeRdf(key, v);
            })
          } else {
            properties += this.writeRdf(key, value[0]);
          }
        } else {
          properties += this.writeRdf(key, value);
        }

      })
      if (!version) properties += "dct:hasVersion \"undefined\";\n"

      data = '\@prefix dcat: <http://www.w3.org/ns/dcat#>.\n\
               @prefix dct: <http://purl.org/dc/terms/>.\n\
               @prefix adms: <http://www.w3.org/ns/adms#> .\n\
               @prefix dqv:  <http://www.w3.org/ns/dqv#> .\n\
               @prefix geodcat: <http://data.europa.eu/930/>.\n\
               @prefix prov: <http://www.w3.org/ns/prov#>. \n\
               @prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>. \n\
               @prefix language: <http://id.loc.gov/vocabulary/iso639-1/>.\n\
               @prefix s: <'+ this.FDP_URL + '/>.\n\
               @prefix c: <'+ this.FDP_URL + '/catalog/>.\n\
               \n\
               s:new\n\
               a dcat:Dataset, dcat:Resource;\n\
               dct:isPartOf c:'+ this.catalogId + ';\n' + properties + '.';

      /*let query = 'query=PREFIX dcat: <http://www.w3.org/ns/dcat#>\n\
               PREFIX dct: <http://purl.org/dc/terms/>\n\
               SELECT ?dataset\n\
               where {\n\
                      ?dataset a dcat:Dataset;\n\
                      dct:title "' + title + '" ;\n\
                      dct:isPartOf <https://f2ds.eosc-pillar.eu/catalog/' + this.catalogId + '>. } '*/


      let requestPromise = this.dataSetService.createDataset(data).then((resp: HttpResponse<any>) => {
        if (resp.status.toString().startsWith('2')) {
          postedDatastes.push(this.ids[i]);
        } else {
          notPostedDatasets.push(this.ids[i]);
        }
      })
      requestPromises.push(requestPromise);
    }
    Promise.all(requestPromises).finally(() => {
      this.loading = false;
      this.dialog.open(FeedbackDialogComponent, {
        data: {
          postedMetadatas: postedDatastes,
          notPostedMetadatas: notPostedDatasets
        }
      }).afterClosed().subscribe(() => this.route.navigateByUrl('/dashboard'));

    })
  }

  private writeRdf(key: string, value: string): string {
    let properties = "";
    (value && typeof value === "string") ? value = this.replaceAll(value, '\n', '') : value;

    switch (key) {
      case 'dcat:landingPage':
        properties += key + ' <' + value + '>;\n';
        break;
      case 'dcat:theme':
        properties += key + ' <' + value + '>;\n';
        break;
      case 'dcat:contactPoint':
        properties += key + ' <' + value + '>;\n';
        break;
      case 'dct:language':
        properties += key + ' <' + value + '>;\n';
        break;
      case 'dct:issued':
        properties += key + ' "' + value + '"^^xsd:dateTime;\n';
        break;
      case 'dct:modified':
        properties += key + ' "' + value + '"^^xsd:dateTime;\n';
        break;
      default:
        properties += key + ' "' + value + '";\n';
        break;
    }
    return properties;
  }
  private replaceAll(str: string, find: string, replace: string) {
    if (str.includes(find)) {
      const strT = str.replace(find, replace)
      this.replaceAll(strT, find, replace)
    } else {
      return str;
    }
  }

  /* function to get recursively all the keys and values from the JSON object */


  getKeysFromMetadataCustom(obj: Object, keyParent: string) {
    Object.keys(obj).forEach(key => {
      if (typeof obj[key] === 'object') {
        if (Array.isArray(obj[key])) {   
          obj[key].forEach(e => {
            if (typeof e === 'object') {
              Object.keys(e).forEach(k => {
                this.keys.push(keyParent + k);
              })
              this.getKeysFromMetadataCustom(e, keyParent + key + ' : ');
            }
          });
          this.keys.push(keyParent + key )
          this.getKeysFromMetadataCustom(obj[key], keyParent + key + ' : ');
        } else {
          this.keys.push(keyParent + key )
          this.getKeysFromMetadataCustom(obj[key], keyParent + key + ' : ');
        }
      } else {

        this.keys.push(keyParent + key );

      }
    });
  }

  getKeysFromMetadataDataverse(obj: Object, keyParent: string) {

    Object.keys(obj).forEach(key => {
      if (typeof obj[key] === 'object') {
        if (Array.isArray(obj[key])) {
          if (keyParent) {
            this.keys.push(keyParent + ' : ' + key)
          } else {
            this.keys.push(key)
          }
          obj[key].forEach(e => {
            if (typeof e === 'object') {
              this.getKeysFromMetadataDataverse(e, keyParent + ' : ' + key + ' : ' + obj[key].indexOf(e));
            } else {
              this.keys.push(keyParent + ' : ' + key + ' : ' + obj[key].indexOf(e))
            }
          });
        } else {
          if (keyParent) {
            this.getKeysFromMetadataDataverse(obj[key], keyParent + ' : ' + key);
          } else {
            this.getKeysFromMetadataDataverse(obj[key], key);
          }
        }
      } else {
        if (keyParent) {
          this.keys.push(keyParent + ' : ' + key)
        } else {
          this.keys.push(key)
        }
      }
    });
  }

  private getValueCustom(tab: string[], item: Object) {
    let obj: Object;
    if (tab.length == 1) {
      return item[tab[0]];
    } else {
      for (let i = 0; i < tab.length; i++) {
        if (i == 0) {
          obj = item[tab[i]]
        } else if (i < tab.length - 1 && obj[tab[i]]) {
          obj = obj[tab[i]];
        } else {
          if (Array.isArray(obj) && obj.some(e => typeof e === 'object')) {
            let array = [];
            obj.forEach(e => {
              if (e[tab[i]]){
                array.push(e[tab[i]]);
              }
            });
            return array;
          } else if (typeof obj === 'object' && Object.values(obj).every(e => Array.isArray(e))) {
            let array = [];
            for (let element in obj) {
              obj[element].forEach(e => {
                if (e[tab[i]]){
                  array.push(e[tab[i]]);
                }
              })
            }
            return array;
          } else if (Array.isArray(obj[tab[i]])) {
            return Object.values(obj[tab[i]]);
          } else {
            return obj[tab[i]];
          }
        }
      }
    }
  }

  private getValueDataverse(tab: string[], item: Object): string {
    let obj: Object
    for (let i = 0; i < tab.length; i++) {
      if (tab.length == 1) {
        return item[tab[0]];
      } else {
        if (i == 0) {
          obj = item[tab[i]]
        } else if (i < tab.length - 1 && obj[tab[i]]) {
          obj = obj[tab[i]];
        } else {
          if (obj[tab[i]] && typeof obj[tab[i]] !== 'object') {
            return obj[tab[i]];
          } else {
            if (i == tab.length - 2) {
              this.getKeysFromMetadataDataverse(item, "");
              console.log(tab[i] + "  " + i)
              this.keys.forEach(k => {
                let table = k.split(" : ");
                if (table[table.length - 2] === tab[i]) {
                  for (let i = 0; i < tab.length; i++) {
                    if (table.length == 1) {
                      return item[tab[0]];
                    } else {
                      if (i == 0) {
                        obj = item[table[i]]
                      } else if (i < table.length - 1 && obj[table[i]]) {
                        obj = obj[table[i]];
                      } else {
                        if (obj[table[i]] && typeof obj[tab[i]] !== 'object') {
                          return obj[table[i]];
                        } else {
                          return 'undefined';
                        }
                      }
                    }
                  }
                }
              })
            }
          }
        }
      }
    }
  }

  isArray(item: any): boolean {
    return Array.isArray(item);
  }

  mapDataset() {
    this.loadingCr = true;
    this.mappedMetadatas = [];
    this.itemsdataset.forEach((dataset: Object) => {
      this.mappedMetadatas.push(this.createDataset(dataset));
    })
    console.log(this.mappedMetadatas);
    this.first = false;
  }
  next() {
    this.loadingCr = true;
    if (this.index < this.itemsdataset.length) {
      this.index += 1;
      this.createDataset(this.itemsdataset[this.index])
      console.log('index = ' + this.index);
    }
  }
  prev() {
    this.loadingCr = true;
    if (this.index > 0) {
      this.index -= 1;
    }
    this.createDataset(this.itemsdataset[this.index])
    console.log('index = ' + this.index);
  }

  downloadJson() {
    let datasetPathArray: DatasetPath[] = [];
    for (let i = 0; i < this.dcatVocabulary.length; i++) {
      if (this.selectedPaths[i]) {
        datasetPathArray.push(new DatasetPath(this.dcatVocabulary[i], this.selectedPaths[i]));
      } else {
        datasetPathArray.push(new DatasetPath(this.dcatVocabulary[i], ""));
      }

    }

    const fileName = 'paths.json';
    const fileType = this.fileSaverService.genType(fileName);
    const txtBlob = new Blob([(JSON.stringify(datasetPathArray)).toString()], { type: fileName });
    this.fileSaverService.save(txtBlob, fileName);
  }

  openFileInputDialog(): void {
    document.getElementById('file').click();
  }

  importJson(event: Event) {
    const files: FileList = (event.target as HTMLInputElement).files;
    const jsonFile = files[0];
    console.log(jsonFile)
    let reader = new FileReader();
    let datasetPath: DatasetPath[] = [];
    reader.onloadend = () => {
      datasetPath = <DatasetPath[]>JSON.parse(reader.result as string);
      for (let i = 0; i < datasetPath.length; i++) {
        this.selectedPaths[i] = datasetPath[i].path;
        if (typeof datasetPath[i].dcatProperty !== 'object') {
          this.dcatVocabulary[i] = this.vocabularies.filter(v => v.identifier === <string>datasetPath[i].dcatProperty)[0];
        } else {
          this.dcatVocabulary[i] = <Dataset>datasetPath[i].dcatProperty;
        }
      }
    }
    reader.readAsText(jsonFile);
  }

  addField(index: number) {
    let addedDcat: Dataset = new Dataset(this.dcatVocabulary[index].name, this.dcatVocabulary[index].identifier, this.dcatVocabulary[index].usageNote, true);
    this.dcatVocabulary.splice(index + 1, 0, addedDcat);
    this.selectedPaths.splice(index + 1, 0, '');
  }

  deleteField(index: number) {
    this.dcatVocabulary.splice(index, 1);
    this.selectedPaths.splice(index, 1);
  }

  isMultiple(vocabulary: string): boolean {
    let bool: boolean = true;
    switch (vocabulary) {
      case "dct:isPartOf":
        bool = false;
        break;
      case "dcat:landingPage":
        bool = false;
        break;
      case "dcat:contactPoint":
        bool = false;
        break;
      case "dct:title":
        bool = false;
        break;
      default:
        bool = true;
        break;
    }
    return bool;
  }

  // to delete a property of dcat dataset mapped
  deleteProperty(key: number) {
    this.mappedMetadatas[this.index].delete(key);
  }

  trackByIndex(index: number, obj: any): any {
    return index;
  }

  /* autocompletion functions */

  private filter(value: string): string[] {
    let filterValue = value.toLowerCase();
    return this.keys.filter(optionValue => optionValue.toLowerCase().includes(filterValue));
  }

  getFilteredOptions(value: string): Observable<string[]> {
    return of(value).pipe(
      map(filterString => this.filter(filterString)),
    );
  }
  onModelChange(value: string) {
    this.filteredOptions = of(this.filter(value));
  }
  reset() {

    this.filteredOptions = of(this.keys);
  }
}







