import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { AppConfiguration } from 'src/app/AppConfiguration';
import { AuthService } from 'src/app/authentication/services/auth.service';
import { environment } from 'src/environments/environment';
import { TokenStorageService } from 'src/app/authentication/services/token-storage.service';

interface PersistentUrlResponse {
  persistentUrl: string;
}

const FDP_URL = environment.fdpUrl;
@Injectable({
  providedIn: 'root'
})
export class PublishRepositoryService {
  fds2Token: string = this.tokenService.getFDPToken();

  constructor(
    private http: HttpClient,
    private appConfig: AppConfiguration,
    private authService: AuthService,
    private tokenService: TokenStorageService
  ) { }

  get smartApiUrl(): string {
    return environment.smartharvesterUrl + '/harvester/api'
  }

  async publishRepository(data: string) {

    if (this.fds2Token) {

      const myHeaders = new Headers();
      myHeaders.append('Accept', 'text/turtle');
      myHeaders.append('Content-Type', 'text/turtle');
      myHeaders.append('Authorization', 'Bearer ' + this.fds2Token);
      const myInit = { method: 'POST', body: data, headers: myHeaders };

      const myRequest = new Request(FDP_URL + "/catalog", myInit);

      const response = await fetch(myRequest, myInit);
      const responseText = await response.text();

      const catId = responseText.substring(38, 74);
      console.log('catalog creaetd with fdp / id = ' + catId);
      return this.addUserCatalog(catId);
    }

    return null;
  }

  getPersistentUrl(): Observable<PersistentUrlResponse> {
    return this.http.get<PersistentUrlResponse>(environment.fdpUrl + '/configs/bootstrap');
  }

  async addUserCatalog(catId: string): Promise<any> {
    const tokenResponse = this.tokenService.getToken();
    const user = this.tokenService.getUser();

    return this.http.post(this.smartApiUrl + '/user/' + user.email + '/catalogs', catId,
      { headers: { Authorization: 'Bearer ' + tokenResponse } })
      .toPromise();
  }

}
